//-*-c++-*-
#ifndef INCLUDED_SimpleChaseBallBehavior_h_
#define INCLUDED_SimpleChaseBallBehavior_h_

#include "Behaviors/BehaviorBase.h"
#include "Motion/MotionManager.h"
#include "Motion/MMAccessor.h"

#include "Events/EventRouter.h"
#include "Events/VisionEvent.h"
#include "Shared/WorldState.h"
#include "Motion/WalkMC.h"
#include "Vision/Vision.h"

//! A simple behavior to chase after any objects seen by the vision system
/*! Similar to ChaseBallBehavior, but this one doesn't try to move the head, so
 *  it's a little more... simple.  However, it does make sure to take into account
 *  which direction the head is pointing when it sees the object. */
class SimpleChaseBallBehavior : public BehaviorBase {
public:
	//!constructor
	SimpleChaseBallBehavior()
		: BehaviorBase(), walker_id(MotionManager::invalid_MC_ID)
	{}
	//!destructor
	virtual ~SimpleChaseBallBehavior() {}

	//! adds a headpointer and a walker, and a listens for vision events
	virtual void DoStart() {
		BehaviorBase::DoStart();
		walker_id = motman->addMotion(SharedObject<WalkMC>());
		erouter->addListener(this,EventBase::visionEGID);
	}

	//! removes motion commands and stops listening
	virtual void DoStop() {
		erouter->forgetListener(this);
		motman->removeMotion(walker_id);
		BehaviorBase::DoStop();
	}

	//! sets the head to point at the object and sets the body to move where the head points
	virtual void processEvent(const EventBase& event) {
		if(event.getGeneratorID()==EventBase::visionEGID && event.getTypeID()==EventBase::statusETID) {
			//in case the head isn't pointing straight forward, we'll straife
			float x=120.0f*cos(state->outputs[HeadOffset+PanOffset]);
			float y=120.0f*sin(state->outputs[HeadOffset+PanOffset]);
			float z=-static_cast<const VisionEvent*>(&event)->getCenterX();
			MMAccessor<WalkMC>(walker_id)->setTargetVelocity(x,y,z);
		}
	}
			
	virtual std::string getName() const { return "SimpleChaseBallBehavior"; } 

protected:
	MotionManager::MC_ID walker_id;      //!< a WalkMC object
};

/*! @file
 * @brief Describes SimpleChaseBallBehavior, which runs around after whatever the dog sees
 * @author ejt (Creator)
 *
 * $Author: ejt $
 * $Name: tekkotsu-1_5 $
 * $Revision: 1.4 $
 * $State: Rel $
 * $Date: 2003/07/28 18:39:18 $
 */

#endif
