//-*-c++-*-
#ifndef INCLUDED_WMMonitorBehavior_h_
#define INCLUDED_WMMonitorBehavior_h_

#include <iostream>
#include "Wireless/Wireless.h"
#include "Behaviors/BehaviorBase.h"
#include "Motion/MotionManager.h"
#include "Events/EventRouter.h"
#include "Events/EventBase.h"
#include "Shared/RobotInfo.h"
#include "Shared/Serializer.h"
#include "Behaviors/Controller.h"
#include "Shared/WMclass.h"

//! gets input from the GUI
int wmmonitorcmd_callback(char *buf, int bytes);
class WMMonitorBehavior;

//! so wmmonitorcmd_callback knows where to send the input from the GUI
extern WMMonitorBehavior *wmMonitorBehavior;

class WMitem_base;

//! Listens to wmmonitor control commands coming in from the command port.
class WMMonitorBehavior : public BehaviorBase {
 protected:
	//! The input command stream socket
	Socket *cmdsock;

 private:
	WMMonitorBehavior(const WMMonitorBehavior&); //!< don't call
	WMMonitorBehavior operator=(const WMMonitorBehavior&); //!< don't call

 public:
	//! constructor
	WMMonitorBehavior() :
	  BehaviorBase(),
	  cmdsock(wireless->socket(SocketNS::SOCK_STREAM, 2048, 8192))
	{ wmMonitorBehavior = this;
    wireless->setDaemon(cmdsock); }
	//! destructor
	virtual ~WMMonitorBehavior() { wmMonitorBehavior = NULL; }

	//! processes input from the GUI
  void registerData(const std::string& s);

	//! Finds the item with the corresponding name
  WMitem_base* find (std::string& s);

	//! Writes out the passed information to #cmdsock
  void report (const std::string& var_type,
               const std::string& var_name,
               const std::string& value);

	//! Writes out the passed information to #cmdsock
  void report (const std::string& var_type,
               const std::string& var_name,
               const char* value, int value_length);

	//! Writes out the passed information to #cmdsock
  void report(const char* var_type, int var_type_length,
              const char* var_name, int var_name_length,
              const char* value, int value_length);

	virtual void DoStart() {
		// Behavior startup
		BehaviorBase::DoStart();
		// Turn on wireless
		wireless->setReceiver(cmdsock->sock, wmmonitorcmd_callback);
		wireless->listen(cmdsock->sock, config->main.wmmonitor_port);
		// open gui
		// GUI is opened by separate behavior ViewWMVarsBehavior
		// until bug with clearing wmvars when restarting this is fixed
		erouter->addListener(this,EventBase::wmVarEGID);
	}

	virtual void DoStop() {
//		Controller::closeGUI(getGUIType());
		erouter->forgetListener(this);
		// Close socket; turn wireless off
		wireless->close(cmdsock);
		// Total behavior stop
		BehaviorBase::DoStop();
	}

	virtual void processEvent(const EventBase& e) {
		WMentry * entry = reinterpret_cast<WMentry*>(e.getSourceID());
		std::string s(entry->item_name);
		WMregistry *temp=entry->registry;
		while (temp!=&GlobalWM && temp!=NULL) {
			s=temp->name + "." + s;
			temp=temp->parent;
		}
		report(entry->type_name, s, entry->item->toString());
	}

	virtual unsigned int getPort() const { return config->main.wmmonitor_port; }

	virtual std::string getName() const { return "Watchable Memory Monitor"; } //!< returns name of behavior
	static std::string getClassDescription() { 
		char tmp[20];
		sprintf(tmp,"%d",config->main.wmmonitor_port);
		return std::string("Bidirectional control communication with WMMonitor on port ")+tmp;
	}
};

//! interfaces with wireless to pass information from the gui to the monitor
int wmmonitorcmd_callback(char *buf, int bytes);

/*! @file
 * @brief Defines WMMonitorBehavior, which listens to commands from the Aibo3D gui and shows current state
 * @author alokl (Creator)
 *
 * $Author: ejt $
 * $Name: tekkotsu-1_5 $
 * $Revision: 1.3 $
 * $State: Rel $
 * $Date: 2003/10/10 17:50:10 $
 */

#endif 
